function [xo,Ot,nS]=univar(S,x0,ip,method,Lb,Ub,problem,tol,mxit)
%   Unconstrained optimization using univariate search (Coggins and Gold Section).
%
%   [xo,Ot,nS]=univar(S,x0,ip,method,Lb,Ub,problem,tol,mxit)
%
%   S: objective function
%   x0: initial point
%   ip: (0): no plot (default), (>0) plot figure ip with pause, (<0) plot figure ip
%   method: (0) Coggins (default), (1): Golden Section
%   Lb, Ub: lower and upper bound vectors to plot (default = x0*(1+/-2))
%   problem: (-1): minimum (default), (1): maximum
%   tol: tolerance (default = 1e-4)
%   mxit: maximum number of stages (default = 50*(1+4*~(ip>0)))
%   xo: optimal point
%   Ot: optimal value of S
%   nS: number of objective function evaluations

%   Copyright (c) 2001 by LASIM-DEQUI-UFRGS
%   $Revision: 1.0 $  $Date: 2001/07/04 21:00:05 $
%   Argimiro R. Secchi (arge@enq.ufrgs.br)
 
 if nargin < 2,
   error('univar requires two input arguments');
 end
 if nargin < 3 | isempty(ip),
   ip=0;
 end
 if nargin < 4 | isempty(method),
   method=0;
 end
 if nargin < 5 | isempty(Lb),
   Lb=-x0-~x0;
 end
 if nargin < 6 | isempty(Ub),
   Ub=2*x0+~x0;
 end
 if nargin < 7 | isempty(problem),
   problem=-1;
 end
 if nargin < 8 | isempty(tol),
   tol=1e-4;
 end
 if nargin < 9 | isempty(mxit),
   mxit=50*(1+4*~(ip>0));
 end

 x0=x0(:);
 y0=feval(S,x0);
 
 n=size(x0,1);
 E=eye(n);
 ips=ip;
 
 if ip & n == 2,
   figure(abs(ip));
   [X1,X2]=meshgrid(Lb(1):(Ub(1)-Lb(1))/20:Ub(1),Lb(2):(Ub(2)-Lb(2))/20:Ub(2));
   [n1,n2]=size(X1);
   f=zeros(n1,n2);
   for i=1:n1,
    for j=1:n2,
      f(i,j)=feval(S,[X1(i,j);X2(i,j)]);
    end
   end
   mxf=max(max(f));
   mnf=min(min(f));
   df=mnf+(mxf-mnf)*(2.^(([0:10]/10).^2)-1);
   [v,h]=contour(X1,X2,f,df); hold on;
%   clabel(v,h);
   h1=plot(x0(1),x0(2),'ro');
   legend(h1,'start point');

   if ip > 0,
     ips=ip+1;
     disp('Pause: hit any key to continue...'); pause;
   else
     ips=ip-1;
   end
 end
 
 x=x0;
 xo=x0;
 Ot=y0;
 it=0;
 nS=1;
 while it < mxit,
  for i=1:n,
   if method,           % to see the linesearch plot, remove the two 0* below
     [stepsize,xo,Ot,nS1]=aurea(S,x,E(:,i),0*ips,problem,tol,mxit);
   else
     [stepsize,xo,Ot,nS1]=coggins(S,x,E(:,i),0*ips,problem,tol,mxit);
   end

   if ip & n == 2,
     figure(abs(ip));
     plot([x(1) xo(1)],[x(2) xo(2)],'r');
     legend(h1,'start point');
     if ip > 0,
       disp('Pause: hit any key to continue...'); pause;
     end
   end

   x=xo;
   nS=nS+nS1;
  end

  it=it+1;
  
  if n==1 | (norm(xo-x0) < tol*(0.1+norm(x0)) & abs(Ot-y0) < tol*(0.1+abs(y0))),
    break;
  end

  y0=Ot;
  x0=xo;
 end

 if it == mxit,
   disp('Warning Univar: reached maximum number of stages!');
 elseif ip & n == 2,
   figure(abs(ip));
   h2=plot(xo(1),xo(2),'r*');
   legend([h1,h2],'start point','optimum');
 end
 